@echo off
rem
rem $Id: pack.bat,v 0.00 2006/11/29 00:00:00 modalsist Exp $
rem
rem This batch file was created to generate zipped files for released version
rem of xharbour binaries, sources, contrib and tests.
rem
rem The syntax is: pack /<platform> /<compiler>
rem For example, to pack binaries: pack /w32 /bcc32 
rem              to pack sources : pack /src
rem
rem This batch requires the free 7-Zip (command line version) for compression.
rem See at: http://www.7-zip.org
rem
rem Adjust the sets below for your ones.

rem set the xharbour's released version.
set hb_ver=0.99.70

rem set the 7-Zip (command line version) path.
set hb_zip_bin=c:\7-zip\7za.exe

set hb_slash=\


REM *************
REM *** START ***
REM *************

if "%1" == ""   goto help
if "%1" == "/?" goto help
if "%1" == "?"  goto help
if "%1" == "-?" goto help

if not exist %hb_zip_bin% goto warning

goto zip_sets

REM ********************
REM *** ZIP Settings ***
REM ********************

:zip_sets

set hb_zip_ext=.zip
set hb_zip_flags=a -bd -tzip -x!*.obj -x!*.log -x!*%hb_zip_ext%
set hb_zip_flags1=a -bd -tzip -r -x!*.obj -x!*.log -x!*%hb_zip_ext%

if "%1" == "/src"   goto pack_src

if "%1" == "/w32"   goto windows
if "%1" == "/linux" goto linux
if "%1" == "/os2"   goto os2
if "%1" == "/dos"   goto dos

goto help

REM ************************
REM *** PLATFORM OPTIONS ***
REM ************************

:windows

set hb_architecture=w32

if "%2" == "/bcc32"    goto borland
if "%2" == "/msvc"     goto msvc
if "%2" == "/pellesc"  goto pelles
if "%2" == "/mingw"    goto mingw
goto help


:linux

set hb_architecture=linux
set hb_compiler=gcc
set hb_slash=/


set hb_zip_ext=.tar.gz
set hb_zip_flags=a -ttar -x!*.o -x!*.log -x!*%hb_zip_ext%
set hb_zip_flags1=a -ttar -r -x!*.o -x!*.log -x!*%hb_zip_ext%
set hb_zip_flags2=a -ttar -r -x!*.o -x!*.log -x!*%hb_zip_ext%

goto pack_bin_linux


:os2
echo *** Need to be configured ***
goto exit

:dos
echo *** Need to be configured ***
goto exit


REM ************************
REM *** COMPILER OPTIONS ***
REM ************************

:borland
set hb_compiler=bcc32
goto pack_bin_w32

:msvc
set hb_compiler=msvc
goto pack_bin_w32

:pelles
set hb_compiler=pellesc-4.50
goto pack_bin_w32

:mingw
set hb_compiler=mingw
goto pack_bin_w32




REM *******************************
REM *** BINARIES FOR MS-WINDOWS ***
REM *******************************

:pack_bin_w32

set hb_filename=xharbour-%hb_ver%.bin.%hb_architecture%.%hb_compiler%%hb_zip_ext%

if exist %hb_filename% del %hb_filename%

%hb_zip_bin%  %hb_zip_flags%   %hb_filename%  bin%hb_slash%harbour.exe
%hb_zip_bin%  %hb_zip_flags%   %hb_filename%  bin%hb_slash%hbmake.exe
%hb_zip_bin%  %hb_zip_flags%   %hb_filename%  bin%hb_slash%harbour.dll
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  doc%hb_slash%*.txt
%hb_zip_bin%  %hb_zip_flags%   %hb_filename%  lib%hb_slash%*.lib
%hb_zip_bin%  %hb_zip_flags%   %hb_filename%  include%hb_slash%*.*

goto exit

REM **************************
REM *** BINARIES FOR LINUX ***
REM **************************

:pack_bin_linux
echo *** Need to be configured ***

goto exit


REM **********************
REM *** SOURCE PACKAGE ***
REM **********************

:pack_src

set hb_filename=xharbour-%hb_ver%.src%hb_zip_ext%

if exist %hb_filename% del %hb_filename%

%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  Changelog
%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  Copying
%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  Todo
%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  MakeFile
%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  *.*

%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  bin%hb_slash%*.sh
%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  bin%hb_slash%*.bat
%hb_zip_bin%  %hb_zip_flags%  %hb_filename%  bin%hb_slash%MakeFile

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  config%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  config%hb_slash%MakeFile

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  debian%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  debian%hb_slash%MakeFile

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  include%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  include%hb_slash%MakeFile

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  samples%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  samples%hb_slash%MakeFile

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  source%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  source%hb_slash%MakeFile

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  utils%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  utils%hb_slash%MakeFile


REM *********************
REM *** Tests package ***
REM *********************

:pack_tests

set hb_filename=xharbour-%hb_ver%.tests%hb_zip_ext%

if exist %hb_filename% del %hb_filename%

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  tests%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  tests%hb_slash%MakeFile


REM ************************
REM *** CONTRIBS PACKAGE ***
REM ************************

:pack_contrib

set hb_filename=xharbour-%hb_ver%.src.contrib%hb_zip_ext%
if exist %hb_filename% del %hb_filename%

%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  contrib%hb_slash%*.*
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  contrib%hb_slash%changelog
%hb_zip_bin%  %hb_zip_flags1%  %hb_filename%  contrib%hb_slash%makefile



REM ***********
REM *** END ***
REM ***********

if exist xharbour-%hb_ver%*.* dir xharbour-%hb_ver%*.*
goto exit

:help
echo.
echo  Use: pack /platform /compiler to pack binaries or
echo       pack /src to pack sources.
echo.
echo  Platform can be: w32, linux, os2, mac, dos  
echo  Compiler can be: bcc32, msvc, gcc, pellesc, mingw
echo.
echo  Example to pack binaries for Windows/Borland: pack /w32 /bcc32 
echo.         to pack sources: pack /src
goto exit

:warning
echo.
echo  %hb_zip_bin% does not exist.
echo  Set the hb_zip_bin environment var properly.
echo.
goto exit

:exit

set hb_ver=
set hb_architecture=
set hb_compiler=
set hb_zip_bin=
set hb_zip_flags=
set hb_zip_flags1=
set hb_zip_ext=
set hb_filename=
set hb_slash=
